/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*	
 * Created on 18.okt.2003
 *
 */

package com.exponto.jstock;
/**
 * Class for handling of user drawn lines.
 * 
 * All lines are stored in the database, and any existing lines can be retrieved from
 * the database. This has to be done manually typically right after calling the 
 * constructor. Lines can, at anytime, be saved to the database. An ArrayList is 
 * used to keep track of the lines.
 * 
 * The lines are stored persistently in the database so they can be correctly drawn
 * if the scale of the chart is changed. Date and price are used to achieve this.
 * 
 * @author kaare
 *
 */

import java.util.ArrayList;
import org.apache.log4j.Logger;

public class Lines {
	ArrayList persistentLines;
	static Logger logger = Logger.getLogger(Lines.class);
//	Line2D.Double currentLine;
	int currentPosition;
	int activeSelection = -1;
	int type;
	int removedNumber = 0;
	TickerID tickerId;
	JstockDB jDB;
	
	/**
	 * Creates a new set of Lines
	 * 
	 * @param id os the  - TickerID that this Lines is associated with
	 * @param isWeekly - if the lines are for a weekly chart or not
	 */
	public Lines(TickerID id, boolean isWeekly) {
//		lines = new ArrayList();
		tickerId = id;
		this.setType(isWeekly);
		persistentLines = new ArrayList();
		currentPosition = 0;
	}


//	protected  void finalize() {
//		// Save lines when they are added or moved iso. 
//		logger.debug("finalize is saving lines on exit");
//		this.saveLines();
//	}
	
//	public void load(int tickerId) {
//	}
	
	/**
	 * Sets the currentPosition to the first line in the ArrayList
	 */
	public void reset() {
		currentPosition = 0;
//		currentLine = (Line2D.Double) lines.get(currentPosistion);
	}
	
	/**
	 * Sets the currentPosition to currentPosition + i.
	 * 
	 * Note: No check of bounds here
	 * 
	 * @param i integer to add to the currentPosition
	 */
	public void changeCurrentLine(int i) {
		currentPosition = currentPosition + i;
	}
	
	/**
	 * Gets the next PersistenLine if there is more.
	 * 
	 * Increments the currentPosition.
	 * 
	 * @return the next PersistentLine if exist, null otherwise.
	 */
	public PersistentLine hasNext() {
		PersistentLine pL = null;
		if (currentPosition < persistentLines.size()) {
			pL = (PersistentLine) persistentLines.get(currentPosition++);
		} 
		return pL;
	}
	/**
	 * Gets the currently active line.
	 * 
	 * @return the PersistentLine currently marked as active.
	 */
	public PersistentLine getActive() {
		return (PersistentLine) persistentLines.get(activeSelection);
	}
	/**
	 * Updates the position of the active line.
	 * 
	 * @param pL the PersistentLine that shall be updated
	 */
	public void setActive(PersistentLine pL) {
		PersistentLine activePL = (PersistentLine) persistentLines.get(activeSelection);
		activePL.setX1(pL.getX1());
		activePL.setY1(pL.getY1());
		activePL.setX2(pL.getX2());
		activePL.setY2(pL.getY2());
//		pL.setDeleted(oldPL.getDeleted());
//		pL.setLineIndex(oldPL.getLineIndex());
//		pL.setSelected(oldPL.getSelected());
//		persistentLines.set(activeSelection, pL);
	}
	
	public void addLine(PersistentLine pL) {
		//lines.add(line);
		persistentLines.add(pL);
	}
	
	public void removeCurrentLine() {
//		lines.remove(currentLine);
//		persistentLines.remove(currentLine);
	}

//	public void moveLine(int x, int y) {
//		PersistentLine pL = (PersistentLine) persistentLines.get(activeSelection);
//		int  select = pL.getSelected();
//	}

/**
 * 
 * @param keepSelected if false all selection is reset. 
 * @return true if a new PersistentLine is selected
 */
	public boolean selectNext(boolean keepSelected) {
		if (!keepSelected) {
			setCurrentSelection(false); 
		}
		if (persistentLines.size() == 0) {
			//No lines to select
			return false;
		}
		//boolean allChecked = false; 
		PersistentLine pL;
		int currentSelection = activeSelection;
		do {
			activeSelection ++;
			if (activeSelection >= persistentLines.size()) {
				activeSelection = 0;
				//allChecked = true;
			}
			pL = (PersistentLine) persistentLines.get(activeSelection);
			if (currentSelection == activeSelection) {
				// Returns without changes if all lines are selected already
				activeSelection = currentSelection;
				return false;
			}
		} while (pL.isSelected() || pL.isDeleted());
		pL.setSelected(true);
		return true;
	}

	/**
	 * 
	 * @param keepSelected if false all selection is reset. 
	 * @return true if a new PersistentLine is selected
	 */
		public boolean selectPrevious(boolean keepSelected) {
			if (!keepSelected) {
				setCurrentSelection(false); 
			}
			//boolean allChecked = false; 
			PersistentLine pL;
			int currentSelection = activeSelection;
			do {
				activeSelection --;
				if (activeSelection < 0) {
					activeSelection = persistentLines.size()-1;
					//allChecked = true;
				}
				pL = (PersistentLine) persistentLines.get(activeSelection);
				if (currentSelection == activeSelection) {
					// Returns without changes if all lines are selected already
					activeSelection = currentSelection;
					return false;
				}
			} while (pL.isSelected());
			pL.setSelected(true);
			return true;
		}

	public void selectNextEnd() {
		PersistentLine pL = (PersistentLine) persistentLines.get(activeSelection);
		int  select = pL.getSelected();
		select ++;
		if (select >= 3) {
			select = -1;
		}
		pL.setSelected(select);
	}

	/**
	 * Deletes the line that is active
	 *
	 */
	public void deleteActiveSelection() {
		if (persistentLines.size() > 0) {
			PersistentLine pL = (PersistentLine) persistentLines.get(activeSelection);
			pL.setDeleted(removedNumber++); 			
		}
		//remove(activeSelection);
	}

	/**
	 * Load lines from the database
	 *
	 */
	public void loadLines() {
//		Save Lines to the database
		jDB = JstockDB.getInstance();
		this.persistentLines = jDB.loadLines(tickerId, type);
		this.reset();
		  
	}
	/**
	 * Saves all lines not marked as deleted into the database.
	 * The lines are read back from the database.
	 *
	 */
	public void saveLines() {
//		Save Lines to the database
		jDB = JstockDB.getInstance();
		this.persistentLines = jDB.saveLines(tickerId, type, persistentLines);
		this.reset();
	}
	
	/**
	 * Get the TickerID associated with the lines.
	 * 
	 * @return the TickerId
	 */
	public TickerID getTickerId() {
		return tickerId;
	}

	/**
	 * The type of chart.
	 * 
	 * @return the type (either daily or weekly)
	 */
	public int getType() {
		return type;
	}

	/**
	 * Sets the TickerID 
	 * @param id The TickerId
	 */
	public void setTickerId(TickerID id) {
		tickerId = id;
	}

	/**
	 * Sets the type of chart theese lines are valid for
	 * 
	 * @param isWeekly boolean that indicates of this is lines for a weekly chart or not.
	 */
	public void setType(boolean isWeekly) {
		if (isWeekly) {
			type = Constants.weeklyLogChart;
		} else {
			type = Constants.dailyLogChart;
		}
	}

	private void setCurrentSelection(boolean select) {
		this.reset();
		PersistentLine pL;
		pL = this.hasNext();
		while (pL != null) {
			pL.setSelected(select);
			pL = this.hasNext();
		}
	}
	
//	public Line2D.Double hasNext() {
//	if (currentPosistion < lines.size()) {
//		currentLine = (Line2D.Double) lines.get(currentPosistion++);
//		return currentLine;
//	} else return null;
//}
//	public void addLine(double x1, double y1, double x2, double y2, PersistentLine pL) {
//	Line2D.Double l = new Line2D.Double(x1, y1, x2, y2);
//	lines.add(l);
//	persistentLines.add(pL);
//}

}
