/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * The series has been reimplemented and it is now possible to set a window
 * in the series using the setCurrentPeriode methodes to define the window.
 * An internal iterator keeps track of the position. 
 * 
 * LogDataSeries.java
 *
 * Created on 01 November 2001, 17:35
 */
package com.exponto.jstock;
/**
 * Handles logaritmic dataseries for a stock. 
 * 
 * Has some methodes for finding certain capabilities for the series.
 *
 * @author  kaare
 * @version 0.5
 */
import java.util.ArrayList;

import com.exponto.util.*;
public class LogDataSeries extends Series {
// TODO: Current implementation is far too unstable as indicators classes 
// needs to look at the whole series, while the candlestickPanel only looks at 
// the last part of it. That's makes it easy to forget to set the view parameters 
// correctly, but I do like the view so I have to find a solution to this.  	
    long maxVol;
    double yMax, yMin = 0.0;
    
    public ArrayList logStock = new ArrayList ();
	private boolean isWeekly;
	private int startOfPeriode = 0;
	private int endOfPeriode = 0;
	
	private int currentPositionInPeriode = 0;
	private LogTradingData currentLTD = new LogTradingData();
	private Date currentDate;
	private TickerID ticker = new TickerID();
	
    int week = 0;
 	//int numberOfWeeks;
    /** 
     * Creates new empty daily LogDataseries 
     * 
     */
    public LogDataSeries() {
		isWeekly = false;
    }
	/** 
	 * Creates new LogDataseries with the isWeekly paramter set
	 * to b.
	 *  
	 * 
	 */
	public LogDataSeries(boolean b) {
		isWeekly = b;
	}
	/**
	 * FOR TEST PURPOSES ONLY!!!
	 * 
	 * Used to easily create a logDataSeries for testing other functions
	 * 
	 * @param closingPrices
	 */
	public LogDataSeries(double [] closingPrices) {
		for (int i = 0; i < closingPrices.length; i++) {
			LogTradingData lTD = new LogTradingData();
			lTD.close = closingPrices[i];
		}
		
	}

	/**
	 * Stores data for a single day at the end of an arraylist. Also insert dummy data (not NULL) if data for
	 * some days between currentDate and lTD.day is missing
	 *  
	 * @param lTD    Trading data to store
	 * 
	 */
	public void storeResult(LogTradingData lTD) {
		// Daily
//		Ignore if lTD is null		
		if (lTD == null) {
			logger.error("Bad input data in LogDataSeries, storeResult: LogTradingData is null, and is not stored, expected date is "+ currentDate);
			//System.out.println("Bad input data in LogDataSeries, storeResult: LogTradingData is null, and is not stored, expected date is "+ currentDate);
			return;			
		}
//		Skip if Saturday (=7) or Sunday (=1) (reason is that it is seen that input 
//		file contains data for a saturday, e.g.  dno ca. 1997 12 16)
		if ((lTD.day.dayInWeek== 1) || (lTD.day.dayInWeek == 7)) {
			logger.error("Bad input data in LogDataSeries, storeResult: LogTradingData is for Saturday or Sunday skipped, date is " + lTD.day);
			//System.out.println("Bad input data in LogDataSeries, storeResult: LogTradingData is for Saturday or Sunday skipped, date is " + lTD.day);
			return;
		}
		lTD.computeLogValues();
		// Check if this is the first element
		if (logStock.isEmpty()) { 
			// and sets this as the currentDate
			currentDate = lTD.day.klone();
		} else {
			// Normal: Get the expected date right
			currentDate.setToPreviousWorkDay();
		}
		
//		It is assumed that if the are not equal, currentDate always will be higher. It is 
//		possible to use the return value from compareDate to verify this to make the code more
//		safe 
		while (currentDate.compareDate(lTD.day)!= 0) {
//			Adds dummy LTD's as long as the dates don't match			
			LogTradingData dummyDay = new LogTradingData(Constants.missing);
			dummyDay.day = currentDate.klone();
			logStock.add(dummyDay);
			//System.out.println("LogDataSeries, storeResult: Dummy tradingData created, date is " + currentDate);
			currentDate.setToPreviousWorkDay();
		}
	   // System.out.println("k.lav var: "+ k.lav + " min er: " + minkurs);
		logStock.add(lTD);
	}

	public void storeResultWeekly(LogTradingData lTD) {
		LogTradingData newWeek = (LogTradingData) lTD.clone();
		logStock.add(newWeek);
	}

	/*
	 * The following methodes is related to that the series itself keeps track of where
	 * it is.
	 */     
    
	/**
	 * Sets the current periode to the whole series 
	 */    
	public void setCurrentPeriode() {
		startOfPeriode = 0;
		endOfPeriode = logStock.size()-1;
		resetCurrentPeriode();
	}

	/**
	 * Sets the current periode to the latest date and numberOfDays back 
	 */    

	public void setCurrentPeriode(int numberOfDays) {
		endOfPeriode = logStock.size()-1;
		startOfPeriode = endOfPeriode - numberOfDays;
		if (startOfPeriode < 0) startOfPeriode = 0;
//		Reset hi's and low's
		resetCurrentPeriode();
	}
	/**
	 * Sets the current periode 
	 */    
   
	public void setCurrentPeriode(int start, int stop) {
		startOfPeriode = start;
		endOfPeriode = stop;
		resetCurrentPeriode();
	}
	public void resetCurrentPeriode() {
		tradedLowInPeriode = Constants.lowStart;
		tradedHighInPeriode = 0.0;
		maxVol = 0L;
		
		moveToStartInCurrentPeriode();
	}
	
	public void moveToStartInCurrentPeriode() {
		currentPositionInPeriode = startOfPeriode;
		currentLTD = this.getResult(currentPositionInPeriode);
		currentDate = currentLTD.day.klone();
		// Sets currentLTD to a non-null empty set
		currentLTD = new LogTradingData();
		// Sets current date to the (working) day before currentdate
		currentDate.setToPreviousWorkDay();
	}

	/**
	 * Finds the next logTradingData in a series.
	 * 
	 * Returns null if somehow the currentPoistion is larger then the endOfPeriode
	 *  
	 * Does not increment currentPositionInPeriode if it returns null, otherwise it does.
	 *  
	 * @return      the next LogTradingData
	 *                
	 */
	
	public LogTradingData getNextLTD () {
		if (currentPositionInPeriode > endOfPeriode) {
//			System.out.println("LogDataSeries, getNextLTD: Sort of out of bounds " + currentDate);
//			Return null at the end of the series	
			return null;
		}
//		Increments the currentDate by one day
		currentDate.setToNextWorkDay();
		 
		LogTradingData tempLTD= this.getResult(currentPositionInPeriode);
		if (tempLTD == null) {
			return null;
		}
		// It is assumed that if the are not equal, currentDate always will be lower
		// Skip the check for weekly data, as the currentDay will never be right
		if (!this.isWeekly) {
			if (currentDate.compareDate(tempLTD.day)!= 0) {
//			Since missing days are filled with dummy lTD when lTDseries are created
				logger.error("LogDataSeries, getNextLTD: No reason to execute this code anymore. currentDate is " + currentDate + " next date is " + tempLTD.day);
				//System.out.println("LogDataSeries, getNextLTD: No reason to execute this code anymore. currentDate is " + currentDate + " next date is " + tempLTD.day);
				return null;
			}
		}
		currentLTD = this.getResult(currentPositionInPeriode++);
		return currentLTD;
	}

	/**
	 * Returns true if there is more elements in a series (window) false if the
	 * currentPosistion is at the last element.
	 *  
	 * Does not increment currentPositionInPeriode 
	 *  
	 * @return  true if there is more elements, false otherwise.     
	 *                
	 */
	public boolean hasNextLTD() {
		boolean result = true;
		if (currentPositionInPeriode > endOfPeriode) {
			result = false;
		} 
		return result;
	}
	
	public int getEndOfPeriode() {
		return endOfPeriode;
	}
	public int getStartOfPeriode() {
		return startOfPeriode;
	}
	/**
	 * 
	 * @return true if the current series is a weekly series
	 */
	public boolean isWeekly() {
		return isWeekly;
	}

/*  Old stuff */
    /**
     * Should be possible to indicate what range the limits shall be calculated from 
     * 
     * Calculates values used to scale the y-axis of the graph. Uses 
     * methods from the super class.
     *
     */
    public void findYScaleLimits(int height) {
        stepSize = stepSize(tradedLowInPeriode, tradedHighInPeriode, height);
        yMin = roundToStepSize(tradedLowInPeriode, stepSize);
        if (tradedLowInPeriode < (yMin + stepSize/2)) yMin = yMin - stepSize; 
        if (yMin <= 0) {
        	// Fix is due to that yMin originally may be negative for certain dataseries.
        	// This may cause some odd values on the y axis, but is better than a exception!!!
        	yMin = Math.floor( tradedLowInPeriode/base(tradedLowInPeriode)) *base(tradedLowInPeriode);
        }
        yMax = roundToStepSize(tradedHighInPeriode, stepSize) + stepSize;
        //System.out.println(" StepSize: "+ stepSize);
    }   
    
	/**
	 * Retrieves data for a day. 
	 * Aviod using this if possible.
	 *
	 * @param index    Index in the arraylist of the wanted day
	 * @return         LogTradingData for the day.
	 */
	public LogTradingData getResult(int index) {
	// Henter data for dag nr <index> i listen
	// Gir IndexOutOfBoundsException hvis index er "feil". Skal sendes videre
		try {
			return (LogTradingData) logStock.get(index);
		} catch (IndexOutOfBoundsException e) {
			logger.error(e.toString());
			return null;
		}
	}
	/**
	 * Finds the position in the current periode to the LogTradingData with
	 * a date that is equl to the date d.
	 *  
	 * @param d Date to look for
	 * @return position in current periode
	 */
	public int findNumberInPeriode(Date d) {
		moveToStartInCurrentPeriode();
		LogTradingData t;
		int i = 0;
		while (hasNextLTD()) {
			t = getNextLTD();
			if (d.compareDate(t.day) == 0) {
				return i; 
			}
			i++; 
		}
		return -1;
	}
	
	public long getMaxVol() {
		return maxVol;
	}
	/**
	 * Rotates the elements in this ArrayList so that the first becomes
	 * the last an vice versa.
	 *  
	 */    
    
	public void rotate() {
		int index = logStock.size()-1;
		ArrayList aL = new ArrayList();
		for (int i = index; i >= 0; --i) {
			aL.add(logStock.get(i));
		}
		logStock = aL;
	}
/**
 * Finds the number of days of the first week present in this dataseries
 * @return integer between 1 and 7 (monday to sunday)
 */	
	public int numberOfDaysInFirstWeek() {
		// This methode does not change the currentPosistion
		int days = 0;
		LogTradingData tempLTD = this.getResult(startOfPeriode);
		int currentWeek = tempLTD.day.week;
		while (currentWeek == tempLTD.day.week) {
			days ++;
			tempLTD = this.getResult(startOfPeriode+days);	
		}
		return days;	
	}
	
	/**
	 * Searches for the next week number related to the currentPosistion in the in a dataseries.
	 * 
	 * This may seem like an akward way to increment the weeknumber, but it also searches
	 *	for new highs and lows in the periode. Also handles if there are 52 or 53 weeks in a year.
	 * 
	 * @return integer The next weeknumber
	 */
	public int getNextWeekNumber() {
		//int nextDay;
		int previousWeek;
		previousWeek = currentLTD.day.week;
		//logger.debug("Previous week & day" + previousWeek + ", " + currentLTD.day.toDigitString());
		while (previousWeek == currentLTD.day.week) {
//			currentPositionInPeriode ++;
			if (currentPositionInPeriode > endOfPeriode) {
				return 0;
			}
			currentLTD = this.getResult(currentPositionInPeriode++);
//			Looking for new highs and lows, use those for scaling
			if (currentLTD.isTraded()) {
				if (currentLTD.high > tradedHighInPeriode ) {
					tradedHighInPeriode = currentLTD.high;
				}
				if (currentLTD.low < tradedLowInPeriode) {
					tradedLowInPeriode = currentLTD.low;
				}
				if (currentLTD.volume > maxVol) {
					maxVol = currentLTD.volume;
				}
			}
		}
		
		return currentLTD.day.week;
	}

	/**
	 * @return The exchange name
	 */
	public String getExchange() {
		return ticker.getExchangeName();
	}

	/**
	 * @return The ticker name
	 */
	public String getTicker() {
		return ticker.getTickerName();
	}

	/**
	 * @return The TickerId class
	 */
	public TickerID getTickerId() {
		return ticker;
	}


	/**
	 * @param string The exchange name
	 */
	public void setExchange(String string) {
		ticker.setExchangeName(string);
	}

	/**
	 * @param string ticker name
	 */
	public void setTicker(String string) {
		ticker.setTickerName(string);
	}

	public void setTickerId(TickerID id) {
		ticker = id;
	}
	
	public String debugInfo() {
		String s = ticker.toString() + "\nC Size of array:" + logStock.size() + "\nPeriode:" + String.valueOf(startOfPeriode) + " to " + String.valueOf(startOfPeriode) + "\nCurrent posistion: " +String.valueOf(currentPositionInPeriode);
		return s;
	}
/*    public static void main(final String[] args) {
	
	minkurs = 12;
        maxkurs = 132;
        skalering();
        System.out.println(" yMin: " + yMin + " yMaks: "+ yMax);
        minkurs = 0.12;
        maxkurs = 0.99;
        skalering();
        System.out.println(" yMin: " + yMin + " yMaks: "+ yMax);
        
    } */

	/**
	 * @return the logaritmic value of the lowest y value used in the displayed area.
	 */
	public double getYMin() {
		return yMin;
	}

//	/**
//	 * @param d the logaritmic value of the lowest y value used in the displayed area.
//	 */
//	private void setYMin(double d) {
//		yMin = d;
//	}

	/**
	 * @return the logaritmic value of the highest y value used in the displayed area.
	 */
	public double getYMax() {
		return yMax;
	}

//	/**
//	 * @param d the logaritmic value of the highest y value used in the displayed area.
//	 */
//	private void setYMax(double d) {
//		yMax = d;
//	}

}
