/*
 * Created on 26-Nov-2003
 *
 */
package com.exponto.jstock;

import com.exponto.util.*;
//import com.exponto.util.*;

/**
 * Stores the start and endpoints of a line in a scale independent way
 * so that the line can be redrawn when the scale is changed.
 * 
 * x axis is represented by date, y axis by a price value
 * 
 * @author kaare
 *
 */
public class PersistentLine {
	Date x1;
	double y1;
	Date x2;
	double y2;
	// lineIndex is used to store the database index for the specific line
	private int lineIndex = -1;
	private int selected = -1; // -1 not selcted, 0 both ends, 1 first end, 2 second end
	private int deleted = -1;
	/**
	 * 
	 */
	public PersistentLine(Date dx1, double dy1, Date dx2, double dy2) {
		x1 = dx1;
		y1 = dy1;
		x2 = dx2;
		y2 = dy2;
	}
	public PersistentLine(int id, String dx1, double dy1, String dx2, double dy2) {
		lineIndex = id;
		x1 = new Date(dx1);
		y1 = dy1;
		x2 = new Date(dx2);
		y2 = dy2;
	}


	/**
	 * @return the Date representing the x1 coordinate 
	 */
	public Date getX1() {
		return x1;
	}

	/**
	 * @return the Date representing the x2 coordinate
	 */
	public Date getX2() {
		return x2;
	}

	/**
	 * @return a value (double) representing the y1 coordinate
	 */
	public double getY1() {
		return y1;
	}

	/**
	 * @return a value (double) representing the y2 coordinate
	 */
	public double getY2() {
		return y2;
	}

	/**
	 * @param d Date representing the x dimension of the start point
	 */
	public void setX1(Date d) {
		x1 = d;
	}

	/**
	 * @param d Date representing the x dimension of the end point
	 */
	public void setX2(Date d) {
		x2 = d;
	}

	/**
	 * @param d double representing the y dimension of the start point
	 */
	public void setY1(double d) {
		y1 = d;
	}

	/**
	 * @param d double representing the y dimension of the start end
	 */
	public void setY2(double d) {
		y2 = d;
	}

	/**
	 * @return the lineIndex as an int
	 */
	public int getLineIndex() {
		return lineIndex;
	}

	/**
	 * @param i sets the lineIndex to i.
	 */
	public void setLineIndex(int i) {
		lineIndex = i;
	}

	/**
	 * @return true if selected, false otherwise
	 */
	public boolean isSelected() {
		boolean isSelected = false;
		if (selected > -1) {
			isSelected = true;
		}
		return isSelected;
	}

	/**
	 * @param b sets this line to selected if true, false otherwise
	 */
	public void setSelected(boolean b) {
		if (b == false) {
			selected = -1;
		} else {
			selected = 0;	
		}
	}

	public void setSelected(int i) {
		selected = i;
	}
	/**
	 * 
	 */
	public int getSelected() {
		return selected;
	}
	public void setDeleted(int deletedNumber) {
		deleted = deletedNumber;
		this.setSelected(false);
	}
	public int getDeleted() {
		return deleted;
	}
	
	public boolean isDeleted() {
		boolean result = true;
		if (deleted == -1) {
			result = false;
		} 
		return result;
	}
	
	public void undelete() {
		deleted = -1;
	}
}
