/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 06-Jul-2004
 *
 */
package com.exponto.jstock;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;

/**
 * Creates a HashMap with all supported URL modifiers set. 
 * 
 * @author kaare
 *
 */
public class URLModifiers {

	public URLModifiers() {
		
	}
	/**
	 * Creates a HashMap with all supported URL modifiers set.
	 * 
	 * @param ticker the ticker to use
	 * @param exchange the exchange to use
	 * @param numberOfDays the number of days to use
	 * @return a HashMap with all modifiers set
	 */
	public static HashMap getURLModifierHashMap(String ticker, String exchange, int numberOfDays) {
		HashMap modifierMap = new HashMap();
		Calendar calendar = new GregorianCalendar();
		String tickerModifier = "ticker";
		String exchangeModifier = "exchange";
		String lengthModifier = "length";
		String toYearModifier ="YEAR"; // Current year
		String toMonthModifier ="MONTH"; // Current month
		String toDayModifier ="DAY"; // Current day
		String fromYearModifier ="year";
		String fromMonthModifier ="month";
		String fromDayModifier ="day";
		
		modifierMap.put(tickerModifier, ticker);
		modifierMap.put(exchangeModifier, exchange);
		modifierMap.put(lengthModifier, String.valueOf(numberOfDays));
		modifierMap.put(toYearModifier, String.valueOf(calendar.get(Calendar.YEAR)));
		// Month from calendar starts with 0 (for january)
		modifierMap.put(toMonthModifier, String.valueOf(calendar.get(Calendar.MONTH) ));
		modifierMap.put(toDayModifier, String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
		//System.out.println("To date: " + calendar.get(Calendar.YEAR) + (calendar.get(Calendar.MONTH) + 1) + calendar.get(Calendar.DAY_OF_MONTH));
		// A rough estimate for setting the from date. numberOfDays is based on 
		//trading days (i.e. 5 days a week)
		int days = (numberOfDays/5 + 1) *7;
		calendar.add(Calendar.DATE, - days); 
		modifierMap.put(fromYearModifier, String.valueOf(calendar.get(Calendar.YEAR)));
		// Month from calendar starts with 0 (for january)
		modifierMap.put(fromMonthModifier, String.valueOf(calendar.get(Calendar.MONTH) ));
		modifierMap.put(fromDayModifier, String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
		//System.out.println("From date: " + calendar.get(Calendar.YEAR) + (calendar.get(Calendar.MONTH) + 1) + calendar.get(Calendar.DAY_OF_MONTH));
		return modifierMap;
	}
	private void initHashMap() {
		long time = System.currentTimeMillis();
		Calendar calender = new GregorianCalendar();
		
	}
	
}
