/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 10-Mar-2004
 *
*/
package com.exponto.jstock.indicators;
import java.util.HashMap;
import org.apache.log4j.Logger;
//import java.util.Collection;
import java.util.Set;
/**
 * This class registeres all available indicators and loads them upon request.
 * Currently the indicators have to be manually added. It's no big deal to read 
 * them from someplace else. 
 * 
 * This is a pure "static" class, no instance of it exists, never ever (at least for 
 * now).
 * 
 * @author kaare
 *
 */
public class IndicatorMaster {
	
	static Logger logger = Logger.getLogger(IndicatorMaster.class);
	/**
	 * this HashMap contains mapping between indicator names and classes
	 * with the implementation of the indicator. To add a new indicator, update 
	 * the HashMap (see the static part a few lines down the code) and add it to the Constants class. 
	 * 
	 * Note that the spelling have to be the same.
	 * 
	 */	
	private static HashMap stringToClass = new HashMap();
	private static String pack = "com.exponto.jstock.indicators."; // The package part of the name (to save a few bytes ...)
	/**
	 * Add new indicators here
	 */
	static {
		stringToClass.put("ma", "MA");
		stringToClass.put("ema", "EMA");
		stringToClass.put("macd", "Macd");
		stringToClass.put("rsi", "Rsi");
//		stringToClass.put("Stochastics", "stochastics");
	}
	
	/**
	 * Query for a list of all supported indicators. The names returned shall
	 * have "display-to-end-user quality".
	 * 
	 * @return a list of all known indicators
	 */
	
	public static String[] getIndicatorsList() {
		Set set;
		set = stringToClass.keySet();
		String [] strA = new String [set.size()];
//		logger.debug(set.toString());
		set.toArray(strA);
		//logger.debug(strA[0]+ strA[1]);
		return strA;
	}
	/**
	 * Creates and returns an empty indicator of the type given by the 
	 * indicatorName.
	 * 
	 * The indicator needs to be initiated before it is used
	 *   
	 * @param indicatorName
	 * @return The created indicator
	 */
	
	public static Indicator getIndicator(String indicatorName) {
		Indicator ind = getIndicatorClass(indicatorName);
		return ind;
	}
	
	private static String getClassName(String displayName) {
		String s1 = displayName.toLowerCase();
		String s = (String) stringToClass.get(s1);
		return pack + s;
	}
	
	private static Indicator getIndicatorClass(String indicatorName) {
		Indicator ind = null;
		Class c = null;
		String subClassName = getClassName(indicatorName);
		try {
			c = Class.forName(subClassName);
			ind = (Indicator) c.newInstance();
		} catch (Exception e) {
			logger.error(e.toString());
			e.printStackTrace();
			logger.error("Unable to load class " + subClassName + " Check if class exist and/or spelling (also in Constants class).");
		}
		return ind;
	}
}
