/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 30-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import com.exponto.jstock.LogDataSeries;
import java.util.ArrayList;
import org.apache.log4j.Logger;

/**
 * Calculates the MACD for a LogDataSeries with given parameters
 * 
 * @author kaare
 *
 */
public class MacdAlgorithm {
	static Logger logger = Logger.getLogger(MacdAlgorithm.class);	
	
	/**
	 * Calculates the MACD over a periode.  
	 * 
	 * @param logDS the logDataSeries 
	 * @param fast the number of days of the fast EMA
	 * @param slow the number of days of the slow EMA
	 * @param trigger the number of days of the trigger EMA
	 * @return an ArrayList with MACD(slow/fast/trigger) data
	 */
	public static ArrayList computeMacd(LogDataSeries logDS, MacdParameters macdParameters) {
		ArrayList macdList = new ArrayList();
		int [] macdValues = macdParameters.getParameters();
		int fast = macdValues[0];
		int slow = macdValues[1];
		int trigger = macdValues[2];
		ArrayList slowEMA = EMAAlgorithm.computeEMA(logDS, slow);
		ArrayList fastEMA = EMAAlgorithm.computeEMA(logDS, fast);
		ArrayList macdTemp = new ArrayList();
		ArrayList triggerEMA;
		int length;
		//int startAt;
		//logger.debug("Length of slow & fast EMA: " + slowEMA.size() + ", " + fastEMA.size());
		
		double s, f;
		Double tempS, tempF, tempT;
		if ((slowEMA != null) && (fastEMA != null) ) {
			// Check that length is equal
			length = slowEMA.size() ;
			if (length != fastEMA.size())  {
//				This should not happend, throw exception iso return null?
				logger.error("EMA used to find MACD has different length");
				return null;
			}	
			for (int i = 0; i < length; i++) {
				// Calculate MACD and put in ArrayList
				tempS = (Double) slowEMA.get(i);
				tempF = (Double) fastEMA.get(i);
				if ((tempS != null) && (tempF != null)) {
					s = tempS.doubleValue();
					f = tempF.doubleValue();
					macdTemp.add(new Double(f - s));
				} else {
					macdTemp.add(null);
				}
			}
			// Find EMA(trigger) of MACD
			triggerEMA = EMAAlgorithm.computeEMA(macdTemp, trigger);
			// Put all data in an ArrayList
			for (int i = 0; i < macdTemp.size(); i++) {
				MacdData macdData;
				tempS = (Double)macdTemp.get(i);
				tempT = (Double) triggerEMA.get(i);
				if ((tempS != null) && (tempT != null)) {
					s = tempS.doubleValue();
					macdData = new MacdData(s , tempT.doubleValue());
				} else {
					macdData = null;
				}
				macdList.add(macdData);
			}
		}
		return macdList;
	}
}
