/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Contains MACD specific parameters used in computation of an
 * indicator series
 * 
 * @author kaare
 *
 */
public class MacdParameters extends IndicatorParameters {
	
	int [] macdParameters = new int[3];
	JTextField macdSlowText;
	JTextField macdFastText;
	JTextField macdTriggerText;
	JComponent [][] compArray = null;
	
	
	public MacdParameters(int fast, int slow, int trigger) {
		macdParameters[0] = fast;
		macdParameters[1] = slow;
		macdParameters[2] = trigger;
		compArray = this.initUserInterface();
	}

	public JComponent[][] getUserInterface() {
		if (compArray == null) {
			compArray = this.initUserInterface();
		}
		return compArray;
	}

	public void updateParameters() {
		this.setParameters(0, Integer.parseInt(macdFastText.getText()));
		this.setParameters(1, Integer.parseInt(macdSlowText.getText()));
		this.setParameters(2, Integer.parseInt(macdTriggerText.getText()));            
	}

	/**
	 * @return the parameters used in the MACD calculation
	 */
	public int [] getParameters() {
		return macdParameters;
	}

	public void setParameters(int index, int value) {
		// Set a resonable default
		if (value < 0)  {
			if (index == 0) {
				// Fast default
				value = 12;
			} else if (index == 1) {
				// Slow default
				value = 26;
			} else if (index == 2) {
				// Trigger default
				value = 9;
			}
		} 
		macdParameters[index] = value;
		if (compArray[index][1] != null) {
			// Update JTextField with new values
			JTextField tempField = (JTextField) compArray[index][1];
			tempField.setText(String.valueOf(value));
		}
//		if (macdText != null) {
//			macdText.setText(String.valueOf(value));
//		}
	}

	public void setParameters(int [] parameters) {
		macdParameters = parameters;
		for (int index = 0; index < compArray.length; index ++) {
			if (compArray[index][1] != null) {
				// Update JTextField with new values
				JTextField tempField = (JTextField) compArray[index][1];
				tempField.setText(String.valueOf(parameters[index]));
			}
		}
//		if (macdText != null) {
//			macdText.setText(String.valueOf(parameters[0]));
//		}
	}
	
	public String [] getParameterNames() {
		String [] strA = new String [3];
		strA[0] = "MACD.Fast";
		strA[1] = "MACD.Slow";
		strA[2] = "MACD.Trigger";
		return strA;
	}

//	public int getNumberOfDays() {
//		return numberOfDays[0];
//	}

	/**
	 * @param i the number of days used in the RSI calculation
	 */
//	public void setNumberOfDays(int i) {
//		numberOfDays[0] = i;
//		if (macdText != null) {
//			macdText.setText(String.valueOf(i));
//		}
//	}
	
	public boolean drawThis() {
		return this.parametersSet();
	}
	
	public boolean equals (Object o) {
		MacdParameters otherMacd = (MacdParameters) o;
		if (otherMacd == null) return false;
		boolean result = true;
		int [] otherMacdParameters = otherMacd.getParameters();
		for (int index = 0; index < this.macdParameters.length; index ++) {
			// Check if equal
			if (this.macdParameters[index] != otherMacdParameters[index]) {
				result = false;
			}
		}
		return result;
	}
	
	public int hashCode() {
		int result = 0;
		for (int index = 0; index < this.macdParameters.length; index ++) {
			result = result + this.macdParameters[index] * (index + 1);
		}
		return result;
	}
	
	public String toString() {
		String s ="";
		for (int index = 0; index < this.macdParameters.length; index ++) {
			if (!s.equalsIgnoreCase("")) {
				s = s + "/" + this.macdParameters[index];
			} else {
				s =  s + this.macdParameters[index];
			}
		}
		return s; 
	}
	
	public Object clone() {
		MacdParameters clone = new MacdParameters(
				this.macdParameters[0], this.macdParameters[1], this.macdParameters[2]);
		return clone;
	}
	
	private JComponent[][] initUserInterface() {
		//JComponent [][] compArray;

		JLabel macdFast = new JLabel("MACD Fast");
		macdFastText = new JTextField(3);
		macdFastText.setText(String.valueOf(this.macdParameters[0]));

		String macdFastTip = "Enter the number of days to use as fast EMA in MACD calculation. Use 0 for no MACD.";
		macdFastText.setToolTipText(macdFastTip);
		macdFast.setToolTipText(macdFastTip);

		JLabel macdSlow = new JLabel("MACD Slow");
		macdSlowText = new JTextField(3);
		macdSlowText.setText(String.valueOf(this.macdParameters[1]));

		String macdSlowTip = "Enter the number of days to use as slow EMA in MACD calculation. Use 0 for no MACD.";
		macdSlowText.setToolTipText(macdSlowTip);
		macdSlow.setToolTipText(macdSlowTip);
		
		JLabel macdTrigger = new JLabel("MACD Trigger");
		macdTriggerText = new JTextField(3);
		macdTriggerText.setText(String.valueOf(this.macdParameters[2]));

		String macdTriggerTip = "Enter the number of days to use as Trigger/Signal line in MACD calculation. Use 0 for no MACD.";
		macdTriggerText.setToolTipText(macdTriggerTip);
		macdTrigger.setToolTipText(macdTriggerTip);


		compArray = new JComponent[3][];
		compArray[0] = new JComponent[2]; 
		compArray[1] = new JComponent[2]; 
		compArray[2] = new JComponent[2]; 

		compArray[0][0] = macdFast;
		compArray[0][1] = macdFastText;
		compArray[1][0] = macdSlow;
		compArray[1][1] = macdSlowText;
		compArray[2][0] = macdTrigger;
		compArray[2][1] = macdTriggerText;

		return compArray;
	}

	private boolean parametersSet() {
		boolean result = true;
		for (int index = 0; index < macdParameters.length; index ++) {
			// Check that all parameters are set
			if (macdParameters[index] <= 0) {
				result = false;
			}
		}
		return result;
	}

}
