/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
//import java.awt.event.ActionEvent;
//import java.awt.event.ActionListener;

/**
 * Contains RSI specific parameters used in computation of an
 * indicator series
 * 
 * @author kaare
 *
 */
public class RsiParameters extends IndicatorParameters {
	
	int [] numberOfDays = new int[1];
	JTextField rsiText;
	JComponent [][] compArray = null;
	String propertyName = "RSI.NumberOfDays";
	String toolTip = "Enter the number of days to use in RSI calculation. Use 0 for no RSI.";
	String labelName = "RSI";

	
	public RsiParameters(int days) {
		numberOfDays[0] = days;
		//compArray = this.initUserInterface();
	}

	public JComponent[][] getUserInterface() {
		if (compArray == null) {
			compArray = this.initUserInterface();
		}
		return compArray;
	}

	public void updateParameters() {
		try {
			//JTextField jTF = (JTextField) compArray[0][1];
			//logger.debug(" Passed RSI Text is: " + jTF.getText());
			this.setParameters(0, Integer.parseInt(rsiText.getText()));            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read input for RSI");
		}

		//this.setParameters(0, Integer.parseInt(rsiText.getText()));            
	}

	/**
	 * @return the number of days used in the RSI calculation
	 */
	public int [] getParameters() {
		return numberOfDays;
	}

	public void setParameters(int index, int value) {
		// Set a resonable default
		if (value < 0) value = 14;
		numberOfDays[index] = value;
		if (rsiText != null) {
			rsiText.setText(String.valueOf(value));
		}

	}

	public void setParameters(int [] parameters) {
		numberOfDays = parameters;
		if (rsiText != null) {
			rsiText.setText(String.valueOf(parameters[0]));
		}

	}
	
	public String [] getParameterNames() {
		String [] strA = new String [1];
		strA[0] = propertyName;
		return strA;
	}

	public int getNumberOfDays() {
		return numberOfDays[0];
	}

	/**
	 * @param i the number of days used in the RSI calculation
	 */
	public void setNumberOfDays(int i) {
		numberOfDays[0] = i;
		if (rsiText != null) {
			rsiText.setText(String.valueOf(i));
		}
	}
	
	public boolean drawThis() {
		boolean result = false;
		if (this.getNumberOfDays() > 0) {
			result = true;
		} 
		return result;
	}
	
	private JComponent[][] initUserInterface() {
		//JComponent [][] compArray;
		JLabel rsi = new JLabel(labelName);
		rsiText = new JTextField(3);
		rsiText.setText(String.valueOf(this.getNumberOfDays()));

		//String rsiTip = 
			//"Enter the number of days to use in RSI calculation. Use 0 for no RSI.";
		rsiText.setToolTipText(toolTip);
		rsi.setToolTipText(toolTip);

		compArray = new JComponent[1][];
		compArray[0] = new JComponent[2]; 

		compArray[0][0] = rsi;
		compArray[0][1] = rsiText;
//		logger.debug(" RSI ui is initiated");
		return compArray;
	}


}
