/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * ReadNetfondsFile.java
 *
 * Created on 26 February 2003, 21:51
 */

package com.exponto.jstock.io;

/**
 * Reads a data file containing quotes on the format:
 * quote_date	paper	exch	open	high	low	close	volume	value
 * 20031117	KEN		OSE		5.15		5.47	5.15	5.37		1013400	5419010
 *
 * @author  kaare
 */
import java.io.* ;
import org.apache.log4j.Logger;

import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.LogTradingData;
import com.exponto.jstock.TickerNotFoundException;
import com.exponto.jstock.Util;
public class ReadNetfondsFile {
	static Logger logger = Logger.getLogger(ReadNetfondsFile.class);
	private boolean eof = false;
	private BufferedReader reader = null; 
	private String fileName = null;
    
    /** Creates a new instance of ReadNetfondsFile */
	public ReadNetfondsFile()  {
		eof = false;
	}
    
     /**
     * Read data from file and store the result. Currently only " Netfonds"
     * format (or similar) are supported.
     * 
     */   
    public LogDataSeries readSeries (String fN, int numberToGet, int format) throws TickerNotFoundException {
    	fileName = fN;
    	int i = 0;
    	LogTradingData lTD = new LogTradingData();
        LogDataSeries lDS = new LogDataSeries();
        //Character ch = new Character('e');
	    try {
			reader = new BufferedReader (new FileReader (fileName));
            while (true) {
            	i ++;
                String str = this.readString();
				if (i == numberToGet) return lDS;
                if (eof) return lDS;
                // Check if string (may) contain valid data, i.e. longer than 7 
                // characters and that the first character is a digit
                if ((str.length() > 7) && Character.isDigit(str.charAt(0))) {
//                	// A very Q&D way of getting ticker and exchange in lDS
//                	if (i == 8) {
//                		lDS.setExchange(Util.findSubString(str, 2));
//						lDS.setTicker(Util.findSubString(str, 1));
//                	}
                	if (format == 0) {
                		lTD = Util.convertString(str);
                	} else if (format == 1) {
                		lTD = Util.convertYahooString(str);
                	} else {
                		// Sholud throw an exception this is an fairly unrecoverable error
                		logger.error("Unsupported file format, tried to use format = " + format);
                	}
                    //lTD = Util.convertString(str);
                    lDS.storeResult(lTD);
                }
            }
        } 
        catch (FileNotFoundException e) {
        	throw new TickerNotFoundException (" Could not find requested ticker in file " + fileName);
        }
//        return null;
    }
	public final synchronized String readString() {
		String s = "";
		try {
			s = reader.readLine();
	  	}
	  	catch (IOException e)  {
	  		logger.error("IOException, Failed to read string for file" + fileName);
	  		logger.error(e.toString());
			//System.out.println("Klasse ReadNetfondsFile, readString feilet for filen: " + fileName);
	  }
	  if (s == null) {
	  		eof = true;
			s = "";
	  }
	  return s;
	}
    
}
