/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * ReadYahooURL.java
 *
 */

package com.exponto.jstock.obsolete;

/**
 * Obsolete
 * 
 * Read data from online from 
 * http://ichart.yahoo.com/table.csv?s=GNR.TO&a=04&b=7&c=2003&d=05&e=24&f=2004&g=d&ignore=.csv
 * Reads a data file containing quotes on the format:
 * 
 * Date,Open,High,Low,Close,Volume,Adj. Close*
 * 23-Jan-04,6.00,6.08,5.87,6.02,1927000,6.02
 *
 * @author  kaare
 */
import java.io.* ;
import org.apache.log4j.Logger;

import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.LogTradingData;
import com.exponto.jstock.TickerNotFoundException;
import com.exponto.jstock.Util;
public class ReadYahooFile {
	static Logger logger = Logger.getLogger(ReadYahooFile.class);
	private boolean eof = false;
	private BufferedReader reader = null; 
	private String fileName = null;
    
    /** Creates a new instance  */
	public ReadYahooFile()  {
		eof = false;
	}
    
     /**
     * Read data from file and store the result. This one is for "Yahoo"
     * format (or similar).
     * 
     * This is very close to ReadNetfonds file and they should be joined
     * DONE and this class is no obsolete
     */   
    public LogDataSeries readSeries (String fN, int numberToGet) throws TickerNotFoundException {
    	fileName = fN;
    	int i = 0;
    	LogTradingData lTD = new LogTradingData();
        LogDataSeries lDS = new LogDataSeries();
        //Character ch = new Character('e');
	    try {
			reader = new BufferedReader (new FileReader (fileName));
            while (true) {
            	i ++;
                String str = this.readString();
				if (i == numberToGet) return lDS;
                if (eof) return lDS;
                // Check if string (may) contain valid data, i.e. longer than 7 
                // characters and that the first character is a digit
                if ((str.length() > 7) && Character.isDigit(str.charAt(0))) {
                	// Only difference from ReadNetfonds file
                    lTD = Util.convertYahooString(str);
                    lDS.storeResult(lTD);
                }
            }
        } 
        catch (FileNotFoundException e) {
        	throw new TickerNotFoundException (" Could not find requested ticker in file " + fileName);
        }
//        return null;
    }
	public final synchronized String readString() {
		String s = "";
		try {
			s = reader.readLine();
	  	}
	  	catch (IOException e)  {
	  		logger.error("IOException, Failed to read string for file" + fileName);
	  		logger.error(e.toString());
			//System.out.println("Klasse ReadNetfondsFile, readString feilet for filen: " + fileName);
	  }
	  if (s == null) {
	  		eof = true;
			s = "";
	  }
	  return s;
	}
    
}
